import 'package:domain/entity/response/s_g_labour_request_option_entity.dart';
import 'package:domain/entity/response/s_g_labour_request_table_entity.dart';
import 'package:domain/repository/labour_sg_repository.dart';

import '../labour_request_add/labour_request_add_page.dart';
import 'package:domain/entity/response/labour_request_index_entity.dart';
import 'package:get/get.dart';
import 'package:plugin_basic/constants/app_constant.dart';
import 'package:plugin_platform/engine/dialog/dialog_engine.dart';
import 'package:plugin_platform/engine/notify/notify_engine.dart';
import 'package:plugin_platform/engine/toast/toast_engine.dart';
import 'package:plugin_platform/http/dio/dio_cancelable_mixin.dart';
import 'package:plugin_platform/http/http_result.dart';
import 'package:shared/utils/date_time_utils.dart';
import 'package:shared/utils/event_bus.dart';
import 'package:shared/utils/util.dart';
import 'package:widgets/dialog/app_default_dialog.dart';
import 'package:widgets/load_state_layout.dart';
import 'package:widgets/widget_export.dart';

import '../labour_request_edit/labour_request_edit_page.dart';
import '../labour_review_workflow/labour_request_workflow_page.dart';
import 'labour_request_filter.dart';
import 'labour_request_list_state.dart';

class LabourRequestListController extends GetxController with DioCancelableMixin {
  final LabourSGRepository _labourRepository = Get.find();
  final LabourRequestListState state = LabourRequestListState();

  var _curPage = 1;
  var _needShowPlaceholder = true;

  //页面PlaceHolder的展示
  LoadState loadingState = LoadState.State_Success;
  String? errorMessage;

  //刷新页面状态
  void changeLoadingState(LoadState state) {
    loadingState = state;
    update();
  }

  // Refresh 控制器
  final EasyRefreshController refreshController = EasyRefreshController(
    controlFinishRefresh: true,
    controlFinishLoad: true,
  );

  // Refresh 刷新事件
  Future onRefresh() async {
    _curPage = 1;
    fetchLabourRequestList();
  }

  // Refresh 加载事件
  Future loadMore() async {
    _curPage++;
    fetchLabourRequestList();
  }

  // 重试请求
  Future retryRequest() async {
    _curPage = 1;
    _needShowPlaceholder = true;
    fetchLabourRequestList();
  }

  /// 获取列表数据
  Future fetchLabourRequestList() async {
    if (_needShowPlaceholder) {
      changeLoadingState(LoadState.State_Loading);
    }

    // 并发执行两个请求
    var futures = [
      _labourRepository.fetchLabourRequestTable(
        agencyId: state.selectedAgencyId,
        startDate: DateTimeUtils.formatDate(state.selectedStartDate, format: "yyyy-MM-dd"),
        endDate: DateTimeUtils.formatDate(state.selectedEndDate, format: "yyyy-MM-dd"),
        status: state.selectedStatusId,
        outletId: state.selectedDepartmentId,
        curPage: _curPage,
        cancelToken: cancelToken,
      ),
      state.indexOptions == null
          ? _labourRepository.fetchLabourRequestOption(
              cancelToken: cancelToken,
            )
          : Future(() => HttpResult(isSuccess: true).convert(data: state.indexOptions!)),
    ];

    //拿到结果
    var results = await Future.wait(futures);
    var listResult = results[0] as HttpResult<SGLabourRequestTableEntity>;
    var optionResult = results[1] as HttpResult<SGLabourRequestOptionEntity>;

    //选项数据
    if (state.indexOptions == null && optionResult.isSuccess) {
      state.indexOptions = optionResult.data!;
    }

    // 处理数据
    if (listResult.isSuccess) {
      handleList(listResult.data?.rows);
    } else {
      errorMessage = listResult.errorMsg;
      changeLoadingState(LoadState.State_Error);
    }

    // 最后赋值
    _needShowPlaceholder = false;
  }

  // 处理数据与展示的逻辑
  void handleList(List<SGLabourRequestTableRows>? list) {
    if (list != null && list.isNotEmpty) {
      //有数据，判断是刷新还是加载更多的数据
      if (_curPage == 1) {
        //刷新的方式
        state.datas.clear();
        state.datas.addAll(list);
        refreshController.finishRefresh();

        //更新展示的状态
        changeLoadingState(LoadState.State_Success);
      } else {
        //加载更多
        state.datas.addAll(list);
        refreshController.finishLoad();
        update();
      }
    } else {
      if (_curPage == 1) {
        //展示无数据的布局
        state.datas.clear();
        changeLoadingState(LoadState.State_Empty);
        refreshController.finishRefresh();
      } else {
        //展示加载完成，没有更多数据了
        refreshController.finishLoad(IndicatorResult.noMore);
      }
    }
  }

  // 清空搜索条件
  void resetFiltering() {
    state.selectedAgencyId = null;
    state.selectedStartDate = null;
    state.selectedEndDate = null;
    state.selectedStatusId = null;
    state.selectedDepartmentId = null;

    //赋值之后刷新
    refreshController.callRefresh();
  }

  //展示筛选的弹窗
  void showFilterDialog() {
    if (state.indexOptions != null) {
      DialogEngine.show(
        widget: LabourRequestFilter(
          optionResult: state.indexOptions!,
          selectedAgencyId: state.selectedAgencyId,
          selectedStartDate: state.selectedStartDate,
          selectedEndDate: state.selectedEndDate,
          selectedStatusId: state.selectedStatusId,
          selectedDepartmentId: state.selectedDepartmentId,
          onFilterAction: (startDate, endDate, statusId, departmentId, agencyId) {
            state.selectedStartDate = startDate;
            state.selectedEndDate = endDate;
            state.selectedStatusId = statusId;
            state.selectedDepartmentId = departmentId;
            state.selectedAgencyId = agencyId;

            //赋值之后刷新
            refreshController.callRefresh();
          },
        ),
        position: DialogPosition.top,
        animType: DialogAnimation.fade,
      );
    }
  }

  /// 根据ID获取Item对象，用于刷新
  void fetchItemByIdAndRefreshItem(String? requestId) async {
    var result = await _labourRepository.fetchLabourRequestTable(
      requestId: requestId,
      cancelToken: cancelToken,
      curPage: 1,
    );

    //处理数据
    if (result.isSuccess) {
      var data = result.data;
      if (data != null && data.rows.isNotEmpty) {
        final requestItem = data.rows[0];

        //找到当前数据中的此 requestId，并替换对象，再刷新
        var index = state.datas.indexWhere((element) => element.id == requestItem.id);
        if (index >= 0) {
          state.datas[index] = requestItem;
          update();
        }
      }
    } else {
      ToastEngine.show(result.errorMsg ?? "Network Load Error".tr);
    }
  }

  @override
  void onReady() {
    super.onReady();
    fetchLabourRequestList();

    registerEventBus();
  }

  @override
  void onClose() {
    unregisterEventBus();
    state.datas.clear();
    super.onClose();
  }

  // EventBus 的事件接收
  Subscription? subscribe;

  void registerEventBus() {
    subscribe = bus.on(AppConstant.eventLabourRequestRefresh, (arg) {
      var requestId = arg as String;
      if (Utils.isNotEmpty(requestId)) {
        fetchItemByIdAndRefreshItem(requestId);
      } else {
        refreshController.callRefresh();
      }
    });
  }

  void unregisterEventBus() {
    bus.off(AppConstant.eventLabourRequestRefresh, subscribe);
  }

  //跳转到添加页面
  void gotoAddLabourPage() {
    LabourRequestAddPage.startInstance((result) {
      if (result is bool) {
        //添加成功之后刷新
        refreshController.callRefresh();
      }
    });
  }

  //去详情页面
  void gotoDetailPage(SGLabourRequestTableRows data) {
    LabourRequestEditPage.startInstance(2, data.id, null);
  }

  //去编辑页面
  void gotoEditPage(SGLabourRequestTableRows data) {
    LabourRequestEditPage.startInstance(1, data.id, (result) {
      if (result != null) {
        fetchItemByIdAndRefreshItem(data.id.toString());
      }
    });
  }

  //去 status 的工作流页面
  void gotoWorkflowPage(SGLabourRequestTableRows data) {
    SGLabourRequestWorkflowPage.startInstance(data.id.toString());
  }

  //删除labourRequest
  void doDelete(String? requestId, int index) {
    DialogEngine.show(
        widget: AppDefaultDialog(
      title: "Confirmation".tr,
      message: "Are you sure you want to delete this labour request?".tr,
      confirmAction: () {
        _submitDeleteRequest(requestId, index);
      },
    ));
  }

  // 执行 Recall
  void doRecall(String? requestId, int index) async {
    var result = await _labourRepository.recallLabourRequestSG(requestId, cancelToken: cancelToken);

    if (result.isSuccess) {
      //刷新Item
      fetchItemByIdAndRefreshItem(requestId);
    } else {
      ToastEngine.show(result.errorMsg ?? "Network Load Error".tr);
    }
  }

  void _submitDeleteRequest(String? requestId, int index) async {
    var result = await _labourRepository.deleteLabourRequestSG(requestId, cancelToken: cancelToken);

    if (result.isSuccess) {
      NotifyEngine.showSuccess("Successful".tr);

      if (state.datas.length <= 1) {
        refreshController.callRefresh();
      } else {
        state.datas.removeAt(index);
        update();
      }
    } else {
      ToastEngine.show(result.errorMsg ?? "Network Load Error".tr);
    }
  }

}
